<?php

namespace smusatov\bzb2u;

class Printer
{
    private $resourcePrinter;
    private Label $label;

    public function __construct(Label $label, string $device = '/dev/usb/lp')
    {
        if(!file_exists($device)) {
            throw new \Exception('The printer path does not exist');
        }

        if(!is_writable($device)) {
            throw new \Exception('The printer is not writable');
        }

        $this->resourcePrinter = fopen($device, 'w+b');

        if($this->resourcePrinter === false) {
            throw new \Exception('The printer could not be opened');
        }

        $this->label = $label;

        $this->init();
    }

    private function init()
    {
        $this->setCharacterSets();
        $this->setSensorThrough();
        $this->setDarknessLevel(5);
        $this->setPrintSpeed();

        // Mnnn
        fprintf($this->resourcePrinter, '%c%s%02u', 27, 'M', max(min(floor((57 - $this->label->getWidth()) / 2), 28), 0));
        usleep(100);

        // Qnnn
        $constQ = [22 => '032', 23 => '016', 24 => '004', 25 => '184', 26 => '176', 27 => '168', 28 => '160', 29 => '152', 30 => '144', 31 => '136', 32 => '128', 33 => '120', 34 => '112', 35 => '104', 36 => '096', 37 => '088', 38 => '080', 39 => '072', 40 => '064', 41 => '056', 42 => '048', 43 => '040', 44 => '032', 45 => '024', 46 => '016', 47 => '008', 48 => '004', 49 => '384'];
        fprintf($this->resourcePrinter, '%c%s%s', 27, 'Q', $constQ[min(ceil($this->label->getHeight() + $this->label->getGap()), 49)]);
        usleep(100);

        // an
        $this->setMicroMarginTop();
    }

    public function setCharacterSets(int $n = 0): self
    {
        $n = max(0, min(9, $n));

        fprintf($this->resourcePrinter, '%c%s%u', 27, 'I', $n);
        usleep(100);

        return $this;
    }

    public function setSensorThrough(): self
    {
        fprintf($this->resourcePrinter, '%c%s', 27, 'F0');
        usleep(100);

        return $this;
    }

    public function setDarknessLevel(int $n = 6): self
    {
        $n = max(0, min(7, $n));

        fprintf($this->resourcePrinter, '%c%s%u', 27, 'D', $n);
        usleep(100);

        return $this;
    }

    public function setPrintSpeed(int $n = 1): self
    {
        $n = max(1, min(2, $n));

        fprintf($this->resourcePrinter, '%c%s%u', 27, 'S', $n);
        usleep(100);

        return $this;
    }

    public function setMicroMarginTop(int $n = 0): self
    {
        $n = max(0, min(7, $n));

        fprintf($this->resourcePrinter, '%c%s%u', 27, 'a', $n);
        usleep(100);

        return $this;
    }

    public function setSensorReflect(): self
    {
        fprintf($this->resourcePrinter, '%c%s', 27, 'F1');
        usleep(100);

        return $this;
    }

    public function feedForm(): self
    {
        fprintf($this->resourcePrinter, '%c', 12);
        usleep(25000);

        return $this;
    }

    public function forward(int $mm): self
    {
        while($mm > 0) {
            fprintf($this->resourcePrinter, '%c%s%02u', 27, 'f', min($mm, 29));
            usleep(250);

            $mm -= 29;
        }

        return $this;
    }

    public function back(int $mm): self
    {
        while($mm > 0) {
            fprintf($this->resourcePrinter, '%c%s%02u', 27, 'R', min($mm, 29));
            usleep(250);

            $mm -= 29;
        }

        return $this;
    }

    public function printLine(array $line): self
    {
        $line = array_map(fn($bit) => empty($bit) ? 0 : 1, $line);
        $line = array_pad($line, ceil(count($line) / 8) * 8, 0);

        $bytes = array_chunk($line, 8);

        $command = '';
        foreach($bytes as $byte) {
            $command .= chr(bindec(join('', $byte)));

        }

        fprintf($this->resourcePrinter, '%cG%02u%s', 27, count($bytes), $command);
        usleep(925);

        return $this;
    }

    public function printLines(array $lines): self
    {
        foreach($lines as $line){
            $this->printLine($line);
        }

        return $this;
    }

}
